--------------------------------------------------------------
--------------------------------------------------------------
-- Lua RedUnitFunctions
-- Author: Gedemon
-- DateCreated: 7/15/2011 1:28:32 AM
--------------------------------------------------------------
--------------------------------------------------------------

--------------------------------------------------------------
-- units capture tiles
--------------------------------------------------------------
function UnitCaptureTile(playerID, UnitID, x, y)
	local bDebug = false
	local plot = Map.GetPlot(x,y)
	if (plot == nil) then
		return
	end
	
	if ( plot:IsCity() or plot:IsWater() ) then
		return
	end

	-- check here if an unit can't capture a tile and return
	local player = Players[ playerID ]
	local unit = player:GetUnitByID(UnitID)
	local unitClassID = unit:GetUnitClassType()
	if not ( CanCaptureTile(unitClassID) ) then
		return
	end

	local plotKey = GetPlotKey ( plot )
	local ownerID = plot:GetOwner()

	-- If the unit is moving on another player territory...
	if (playerID ~= ownerID and ownerID ~= -1) then
		Dprint("-------------------------------------")
		Dprint("Unit moving on tile ("..x..","..y..") is in another civ (id="..ownerID..") territory", bDebug)

		local player2 = Players[ ownerID ]		
		local team = Teams[ player:GetTeam() ]
		local team2 = Teams[ player2:GetTeam() ]

		-- If we are at war with the other player :
		if team:IsAtWar( player2:GetTeam() ) then
			Dprint(" - Unit owner (id="..playerID..") and tile owner (id="..ownerID..") are at war", bDebug)
			firstOwner = GetPlotFirstOwner( plotKey )
			-- liberating our territory
			if (firstOwner == playerID) then
				Dprint(" - Unit is liberating it's own territory", bDebug)
				local capitalCity = player:GetCapitalCity()
				plot:SetOwner(playerID, capitalCity:GetID() )
			-- capturing current owner territory			
			elseif ( ownerID == firstOwner) then
				Dprint(" - Unit is capturing territory", bDebug)
				local capitalCity = player:GetCapitalCity()
				plot:SetOwner(playerID, capitalCity:GetID() )
			else
				-- don't free old owner territory if we're at war !
				local player3 = Players[ firstOwner ]				
				if team:IsAtWar( player3:GetTeam() ) then
					Dprint(" - Unit is capturing territory, old owner civ (id=".. firstOwner ..") is also at war with unit owner", bDebug)
					local capitalCity = player:GetCapitalCity()
					plot:SetOwner(playerID, capitalCity:GetID() )				
				elseif ( not player3:IsAlive() ) then
					Dprint(" - Unit is capturing territory, old owner civ (id=".. firstOwner ..") is dead", bDebug)
					local capitalCity = player:GetCapitalCity()
					plot:SetOwner(playerID, capitalCity:GetID() )				
				else
				-- liberating old owner territory
					Dprint(" - Unit is liberating this territory belonging to another civ (id=".. firstOwner ..")", bDebug)
					--local capitalCity = player3:GetCapitalCity()
					--if (capitalCity ~= nil) then
					--	plot:SetOwner(firstOwner, capitalCity:GetID() )
					--else						
						plot:SetOwner(firstOwner, -1 )
					--end
				end
			end
		end
	end

end
-- GameEvents.UnitSetXY.Add( UnitCaptureTile )
-- add to Events after map initialization (or re-loading) in RedEuro1940.Lua or other scenario Lua file


--------------------------------------------------------------
-- Units renaming
--------------------------------------------------------------
function UnitName(playerID, unitID, num) -- num = number of unit of this kind
	
	local bDebug = false
	local player = Players[ playerID ]
	local unit = player:GetUnitByID( unitID )

	--Dprint("-------------------------------------")
	Dprint ("Renaming new unit...", bDebug)
	local unitType = unit:GetUnitType()
	local str = nil

	--------------------------------------------- France ---------------------------------------------
	if ( GetCivIDFromPlayerID (playerID) == FRANCE ) then		
		if ( unitType == FR_INFANTRY ) then
			if (num == 1) then
				str = "1ere "
			else			
				str = num .. "eme "
			end
			str = str .. "Division d'infanterie"
			unit:SetName(str)
		end	
		if ( unitType == FR_LEGION ) then
			if (num == 1) then
				str = "1er "
			else			
				str = num .. "eme "
			end
			str = str .. "Regiment etranger d'infanterie"
			unit:SetName(str)
		end
		if ( unitType == FR_TANK ) then
			if (num == 1) then
				str = "1ere "
			else			
				str = num .. "eme "
			end
			str = str .. "Division blindee"
			unit:SetName(str)
		end
		if ( unitType == FR_FIGHTER ) then
			if (num == 1) then
				str = "1er "
			else			
				str = num .. "eme "
			end
			str = str .. "Escadron de chasse"
			unit:SetName(str)
		end
		if ( unitType == FR_BOMBER ) then
			if (num == 1) then
				str = "1er "
			else			
				str = num .. "eme "
			end
			str = str .. "Escadron de bombardement"
			unit:SetName(str)
		end
		if ( unitType == FR_BATTLESHIP ) then
			local name = { "Dunkerque", "Strasbourg", "Richelieu", "Jean Bart"}
			if (num <= # name) then
				str = name[num]
				unit:SetName(str)
			end
		end

	--------------------------------------------- Germany ---------------------------------------------
	elseif ( GetCivIDFromPlayerID (playerID) == GERMANY ) then
 		
		if ( unitType == GE_INFANTRY ) then
			str = num .. ". Infanterie-Division"
			unit:SetName(str)
		end	
		if ( unitType == GE_TANK ) then
			str = num .. ". Panzer-Division"
			unit:SetName(str)
		end
		if ( unitType == GE_FIGHTER ) then
			str = "Jagdgeschwader " .. num
			unit:SetName(str)
		end
		if ( unitType == GE_BOMBER ) then
			str = "Kampfgeschwader " .. num
			unit:SetName(str)
		end
		if ( unitType == GE_BATTLESHIP ) then
			local name = { "Bismark", "Tirpitz", "Hutten", "Berlichingen"}
			if (num <= # name) then
				str = name[num]
				unit:SetName(str)
			end
		end

	--------------------------------------------- Italy ---------------------------------------------
	elseif ( GetCivIDFromPlayerID (playerID) == ITALY ) then
 		
		if ( unitType == IT_INFANTRY ) then
			str = num .. "a Divisione fanteria"
			unit:SetName(str)
		end	
		if ( unitType == IT_TANK ) then
			str = num .. "a Divisione corazzata"
			unit:SetName(str)
		end
		if ( unitType == IT_FIGHTER ) then
			str = num .. "a Squadriglia di caccia"
			unit:SetName(str)
		end
		if ( unitType == IT_BOMBER ) then
			str = num .. "a Squadriglia di bombardieri"
			unit:SetName(str)
		end
		if ( unitType == IT_BATTLESHIP ) then
			local name = { "Vittorio Veneto", "Littorio", "Roma", "Impero"}
			if (num <= # name) then
				str = name[num]
				unit:SetName(str)
			end
		end

	--------------------------------------------- USSR ---------------------------------------------
	elseif ( GetCivIDFromPlayerID (playerID) == USSR ) then
 		
		if ( unitType == RU_INFANTRY ) then
			if (num == 1) then
				str = "1st "
			elseif (num == 2) then			
				str = "2nd "
			elseif (num == 3) then			
				str = "3rd "
			else
				str = num .. "th "
			end
			str = str .. "Rifle Division"
			unit:SetName(str)
		end	
		if ( unitType == RU_TANK ) then
			if (num == 1) then
				str = "1st "
			elseif (num == 2) then			
				str = "2nd "
			elseif (num == 3) then			
				str = "3rd "
			else
				str = num .. "th "
			end
			str = str .. "Tank Corps"
			unit:SetName(str)
		end
		if ( unitType == RU_FIGHTER ) then
			if (num == 1) then
				str = "1st "
			elseif (num == 2) then			
				str = "2nd "
			elseif (num == 3) then			
				str = "3rd "
			else
				str = num .. "th "
			end
			str = str .. "Fighter Aviation Division"
			unit:SetName(str)
		end
		if ( unitType == RU_BOMBER ) then
			if (num == 1) then
				str = "1st "
			elseif (num == 2) then			
				str = "2nd "
			elseif (num == 3) then			
				str = "3rd "
			else
				str = num .. "th "
			end
			str = str .. "Bomber Aviation Division"
			unit:SetName(str)
		end

	--------------------------------------------- other ---------------------------------------------
	-- to do : rename by class ?
	else
 		
		if ( unitType == INFANTRY or unitType == UK_INFANTRY or unitType == GE_INFANTRY ) then
			if (num == 1) then
				str = "1st "
			elseif (num == 2) then			
				str = "2nd "
			elseif (num == 3) then			
				str = "3rd "
			else
				str = num .. "th "
			end
			str = str .. "Infantry Division"
			unit:SetName(str)
		end	
		if ( unitType == TANK or unitType == UK_TANK or unitType == PL_TANK ) then
			if (num == 1) then
				str = "1st "
			elseif (num == 2) then			
				str = "2nd "
			elseif (num == 3) then			
				str = "3rd "
			else
				str = num .. "th "
			end
			str = str .. "Armored Division"
			unit:SetName(str)
		end
		if ( unitType == FIGHTER or unitType == UK_FIGHTER ) then
			if (num == 1) then
				str = "1st "
			elseif (num == 2) then			
				str = "2nd "
			elseif (num == 3) then			
				str = "3rd "
			else
				str = num .. "th "
			end
			str = str .. "Fighter Squadron"
			unit:SetName(str)
		end
		if ( unitType == BOMBER or unitType == UK_BOMBER ) then
			if (num == 1) then
				str = "1st "
			elseif (num == 2) then			
				str = "2nd "
			elseif (num == 3) then			
				str = "3rd "
			else
				str = num .. "th "
			end
			str = str .. "Bomber Squadron"
			unit:SetName(str)
		end
		if ( unitType == UK_BATTLESHIP ) then
			local name = { "Hood", "Anson", "Howe", "Rodney"}
			if (num <= # name) then
				str = name[num]
				unit:SetName(str)
			end
		end
	end
	if str then
		Dprint ("New name : " .. str, bDebug)
		return str
	else
		Dprint ("No name found", bDebug)
		return unit:GetName()
	end
end

--------------------------------------------------------------
-- Units management
--------------------------------------------------------------
function PathBlocked(pPlot, pPlayer)
	if ( pPlot == nil or pPlayer == nil) then
		Dprint ("WARNING : CanPass() called with a nil argument")
		return true
	end

	local ownerID = pPlot:GetOwner()
	local playerID = pPlayer:GetID()

	if ( ownerID == playerID or owner == -1 ) then
		return false
	end

	local pOwner = Players [ ownerID ]
	if ( pPlayer:GetTeam() == pOwner:GetTeam() or pOwner:IsAllies(playerID) or pOwner:IsFriends(playerID) ) then
		return false
	end

	--local team1 = Teams [ pPlayer:GetTeam() ]
	local plotTeam = Teams [ pOwner:GetTeam() ]
	if plotTeam:IsAllowsOpenBordersToTeam( pPlayer:GetTeam() ) then
		return false
	end

	return true
end
function CanGetSupply (playerID, unitID, bShow )

	local bDebug = true
	local player = Players[ playerID ]
	local unit = player:GetUnitByID( unitID )
	local unitPlot = unit:GetPlot()
	local unitType = unit:GetUnitType()
	local unitArea = unitPlot:GetArea()
	local bHighlight = bShow or false

	if ( GameInfo.Units[unitType].Domain == "DOMAIN_SEA" or GameInfo.Units[unitType].Domain == "DOMAIN_AIR" or unit:IsEmbarked() ) then
		-- for now don't check naval supply lines
		return true
	end

	-- first check closest own cities
	Dprint (" - Search supply line for " .. unit:GetName() .. " (unitID =".. unit:GetID() ..", playerID =".. playerID ..")", bDebug)
	local closeCity = GetCloseCity ( playerID, unitPlot )
	local cityPlot = closeCity:Plot()
	-- first check the area, no need to calculate land path if not in the same land... 
	if ( cityPlot:GetArea() == unitArea and isPlotConnected(player, unitPlot, cityPlot, "Land", bHighlight, nil , PathBlocked) ) then
		Dprint ("   - Found path with close city (".. closeCity:GetName() ..")", bDebug )
		return true
	end
	
	-- then all own cities
	--if not player:IsMinorCiv() then -- no city loop for minor !!!!
	Dprint ("   - No path with close city, check all cities", bDebug )
	for city in player:Cities() do
		local cityPlot = city:Plot()
		if ( cityPlot:GetArea() == unitArea and isPlotConnected(player, unitPlot, cityPlot, "Land", bHighlight, nil ,PathBlocked) ) then
			Dprint ("   - Found path with city (".. closeCity:GetName() ..")", bDebug )
			return true
		end	
	end
	--end -- end of test for minor

	if not player:IsMinorCiv() then -- don't test minors for ally/friends
	-- now check friend / ally cities
	for player_num = 0, GameDefines.MAX_CIV_PLAYERS-1, 1 do
		if player_num ~= playerID then
			local other = Players [ player_num ]
			--local unitTeam = Teams [ player:GetTeam() ]
			local bAllied = ( player:GetTeam() == other:GetTeam() or other:IsAllies(playerID) )
			if ( bAllied ) then
				Dprint ("   - No path with close city, check ally (" .. other:GetName() .. ") cities", bDebug)
				-- first check closest allied city
				local closeCity = GetCloseCity ( player_num, unitPlot )
				local cityPlot = closeCity:Plot()
				-- check the area first, no need to calculate land path if not in the same land... 
				if ( cityPlot:GetArea() == unitArea and isPlotConnected(player, unitPlot, cityPlot, "Land", bHighlight, nil , PathBlocked) ) then
					Dprint ("   - Found path with close city (".. closeCity:GetName() ..")", bDebug )
					return true
				end
				
				--if not other:IsMinorCiv() then -- no city loop for minor !!!!
				-- then all his cities
				Dprint ("   - No path with close city, check all " .. other:GetName() .. " cities", bDebug )
				for city in other:Cities() do
					local cityPlot = city:Plot()
					if ( cityPlot:GetArea() == unitArea and isPlotConnected(player, unitPlot, cityPlot, "Land", bHighlight, nil ,PathBlocked) ) then
						Dprint ("   - Found path with city (".. closeCity:GetName() ..")", bDebug )
						return true
					end	
				end
				--end -- end of test for minor
			end
		end
	end	
	end -- end of test for minor
end
function DynamicUnitPromotion(playerID)

	local bDebug = true

	local player = Players[playerID]
	if ( player:IsAlive() ) then	
		Dprint("-------------------------------------")
		Dprint("Add and remove dynamic promotions for ".. player:GetName() .." units ...", bDebug)
		for unit in player:Units() do
			-- Supply line ?
			if CanGetSupply(playerID, unit:GetID()) then
				if (unit:IsHasPromotion(PROM_NO_SUPPLY)) then
					Dprint("   - Removing mark " .. unit:GetName() .. " (unitID =".. unit:GetID() ..", playerID =".. playerID .."), supply line reopen", bDebug)
					unit:SetHasPromotion(PROM_NO_SUPPLY, false)
				end
			elseif not (unit:IsHasPromotion(PROM_NO_SUPPLY)) then
				Dprint("   - Marking " .. unit:GetName() .. " (unitID =".. unit:GetID() ..", playerID =".. playerID .."), has no supply line", bDebug)
				unit:SetHasPromotion(PROM_NO_SUPPLY, true)
			else
				Dprint("   - " .. unit:GetName() .. " (unitID =".. unit:GetID() ..", playerID =".. playerID .."), has still no supply line but is already marked", bDebug)
			end
		end
	end
end
function DynamicTilePromotion(playerID, UnitID, x, y)

	local bDebug = false

	local plot = Map.GetPlot(x,y)
	local player = Players [ playerID ]
	local unit = player:GetUnitByID(UnitID)
	local unitType = unit:GetUnitType()

	if (plot) then 
		Dprint("-------------------------------------", bDebug)
		Dprint("Add and remove dynamic tile promotions for ".. unit:GetName() .." (id=".. unit:GetID() ..")", bDebug)

		if CanParadrop(unitType) then
			if ( plot:IsCity() and plot:GetOwner() == playerID) then
				if not (unit:IsHasPromotion(PROM_PARADROP)) then
					Dprint("   - Marking " .. unit:GetName() .. " (unitID =".. unit:GetID() ..") of ".. player:GetName() ..", can Paradrop", bDebug)
					unit:SetHasPromotion(PROM_PARADROP, true)
				end
			elseif (unit:IsHasPromotion(PROM_PARADROP)) then
				Dprint("   - Marking " .. unit:GetName() .. " (unitID =".. unit:GetID() ..") of ".. player:GetName() ..", can no longer Paradrop", bDebug)
				unit:SetHasPromotion(PROM_PARADROP, false)
			end
		end

		-- Embarked land units get BIG penalty against sea units to compensate higher strength (to protect from one shoot air raid)
		if ( plot:IsWater() and unit:IsEmbarked() ) then

			if not (unit:IsHasPromotion(PROM_PENALTY_SEA)) then
				Dprint("   - Marking " .. unit:GetName() .. " (unitID =".. unit:GetID() ..") of ".. player:GetName() ..", Sea Penalty for embarked unit", bDebug)
				unit:SetHasPromotion(PROM_PENALTY_SEA, true)
			else		
				Dprint("   - " .. unit:GetName() .. " (unitID =".. unit:GetID() ..") of ".. player:GetName() ..", is still embarked but is already marked", bDebug)
			end

		elseif (unit:IsHasPromotion(PROM_PENALTY_SEA)) then
			Dprint("   - Removing mark " .. unit:GetName() .. " (unitID =".. unit:GetID() ..") of ".. player:GetName() ..", has debarked", bDebug)
			unit:SetHasPromotion(PROM_PENALTY_SEA, false)
		end	
	end
end

--------------------------------------------------------------
-- Units placement from Order of Battle
--------------------------------------------------------------
function PlaceUnits(oob)
	-- Place units on 7 plots (central + 6 adjacents)
	local iW, iH = Map.GetGridSize();
	local direction_types = {
			DirectionTypes.DIRECTION_NORTHEAST,
			DirectionTypes.DIRECTION_EAST,
			DirectionTypes.DIRECTION_SOUTHEAST,
			DirectionTypes.DIRECTION_SOUTHWEST,
			DirectionTypes.DIRECTION_WEST,
			DirectionTypes.DIRECTION_NORTHWEST
	}
	local playerID = GetPlayerIDFromCivID( oob.CivID, oob.IsMinor )
	local player = Players [ playerID ]
	local plot = GetPlot(oob.X, oob.Y)
	local direction = 0
	for i, unitType in ipairs (oob.Group) do
		if direction == 0 then
			-- place first unit on central plot
			if plot:GetNumUnits() == 0 then
				local unit = player:InitUnit(unitType, oob.X, oob.Y)
			end
			direction = direction +1;
		else
			-- place through adjacent plots.
			local adjPlot = Map.PlotDirection(oob.X, oob.Y, direction)
			if adjPlot ~= nil then
				local adjX = adjPlot:GetX();
				local adjY = adjPlot:GetY();
				if adjPlot:GetNumUnits() == 0 then
					local unit = player:InitUnit(unitType, adjX, adjY)
				end
			end
			direction = direction +1;
		end
	end
end
function PlaceAirUnits(oob)
	local playerID = GetPlayerIDFromCivID( oob.CivID, oob.IsMinor )
	local player = Players [ playerID ]
	local plot = GetPlot(oob.X, oob.Y)
	for i, unitType in ipairs (oob.Group) do
		local unit = player:InitUnit(unitType, oob.X, oob.Y)	
	end
end

--------------------------------------------------------------
-- Minor Civilizations Units replacing
--------------------------------------------------------------
function ChangeUnitTo ( playerID, unitID, unitType )
	local player = Players[ playerID ]
	local unit = player:GetUnitByID( unitID )

	-- get position before killing the unit !
	local x = unit:GetX()
	local y = unit:GetY()

	-- kill old unit
	unit:SetDamage( unit:GetMaxHitPoints() )

	-- create the new unit from the old one
	local newUnit = player:InitUnit(unitType, x, y)
	newUnit:SetExperience ( unit:GetExperience() )
	newUnit:SetLevel ( unit:GetLevel() )
	-- should be ok to give existing promotions, we're creating UU from standard units, not the other way...
	for unitPromotion in GameInfo.UnitPromotions() do
		local unitPromotionID = unitPromotion.ID
		if( unit:IsHasPromotion( unitPromotionID ) ) then
			newUnit:SetHasPromotion( unitPromotionID )
		end
	end
end
function GetMinorUUType(playerID, unitClassTypeID)
	local player = Players[ playerID ]
	local playerCivID = player:GetMinorCivType()
	local minorUUID = nil
	local minorList = g_Minor_UU[playerCivID]
	if minorList then
		minorUUID = minorList[unitClassTypeID]
	end

	if minorUUID then
		return minorUUID
	end
	return -1
end
function SetMinorUU( playerID, unitID )

	local player = Players[ playerID ]
	if ( player:IsMinorCiv() ) then
		local unit = player:GetUnitByID( unitID )
		local unitType = unit:GetUnitType()		
		local unitClassTypeID = unit:GetUnitClassType()
		local citystateUUTypeID = GetMinorUUType( playerID, unitClassTypeID )

		-- be cautious when setting conditions, remember that we can create an infinite loop here
		if (citystateUUTypeID >= 0  and unitType ~= citystateUUTypeID) then
			ChangeUnitTo ( playerID, unitID, citystateUUTypeID )
			return true
		end
	end

	return false
end

--------------------------------------------------------------
-- Units Initialization
--------------------------------------------------------------

function RegisterNewUnit(playerID, unit) -- unit is object, not ID
	local unitClass = unit:GetUnitClassType()
	local unitType = unit:GetUnitType()
	local value = g_Unit_Default[unitClass]
	local unitID = unit:GetID()

	if ( value == nil ) then
		Dprint("WARNING : unit (id="..unit:GetID()..", class='".. GameInfo.Units[unitType].Type .."', type='".. GameInfo.Units[unitType].Type .."') is not defined in g_Unit_Default table" )
		value = {Moral = 100, NumType = -1, MaterielRatio = 35, MaxHP = 100}
	end	

	if (value.NumType == nil) then
		Dprint("WARNING : NumType for type='".. GameInfo.Units[unitType].Type .."' class=".. GameInfo.Units[unitType].Class .." (".. unitClass ..") is not defined in g_Unit_Default table" )
		-- some units did not have valid NumType but everything else is okay, initialize with unitType as hardcoded fix :-/
		-- todo: find out why this is needed
		value.NumType = unitType
	end

	-- temp, waiting to see if variable maxHP could be implemented
	local maxHP = MAX_HP -- maxHP from Defines here
	--[[
	local maxHP = value.MaxHP
	local initialDamage = MAX_HP - value.MaxHP -- set custom maxHP by substracting the value.MaxHP (requested MaxHit point for this unii) to the global MAX_HP than applying the result as damage (or none if the requested max HP is >= global max HP)
	if initialDamage > 0 then
		unit:SetDamage (initialDamage)
	end 
	--]]

	local materiel = maxHP * value.MaterielRatio / 100
	local personnel = maxHP - materiel
	local unitKey = GetUnitKey(unitID,playerID)

	g_UnitData[unitKey] = { UnitId = unitID, BuilderID = playerID, NumType = value.NumType , BuilderID = playerID, Type = GameInfo.Units[unitType].Type, TypeID = unitType, Moral = value.Moral, MaterielRatio = value.MaterielRatio, Materiel = materiel, MaxMateriel = materiel, Personnel = personnel, MaxPersonnel = personnel, MaxHP = value.MaxHP }

	local num = CountUnitType (playerID, unitClass)
	UnitName(playerID, unitID, num)

end

function InitializeUnit(playerID, unitID)

	local player = Players[ playerID ]
	local unit = player:GetUnitByID( unitID )
	local unitKey = GetUnitKey(unitID,playerID)

	-- initialize only new units...
	if unit:GetGameTurnCreated() ~= Game.GetGameTurn() then
		return
	end

	if ( SetMinorUU(playerID, unitID) ) then
		-- units changed to Minor UU, don't initialize
		return
	end
	
	if g_UnitData[unitKey] then
		-- unit already registered, don't add it again...
		return
	end

	Dprint ("Initializing new unit for ".. player:GetName() .."...")
	RegisterNewUnit(playerID, unit)
	Dprint("-------------------------------------")

end
-- Events.SerialEventUnitCreated.Add( InitializeUnit )
-- add to Events after map initialization (or re-loading) in RedEuro1940.Lua or other scenario Lua file

-- Initialize Order of Battle
function InitializeOOB ()
	-- mass units initialization is made in PlaceUnits() and PlaceAirUnits() functions
	Dprint("-------------------------------------")
	Dprint("Initialize Order of Battle")

	for i, oob in ipairs (g_Initial_OOB) do
		Dprint("-------------------------------------")
		Dprint("Placing ".. oob.Name)
		if oob.Domain == "Land" or  oob.Domain == "Sea" then
			PlaceUnits(oob)
		elseif oob.Domain == "Air" then
			PlaceAirUnits(oob)
		else
			Dprint("WARNING, domain is not valid : ".. oob.Domain)
		end		
	end
	Dprint("-------------------------------------")
	Dprint("End of Order of Battle initialization ...")
end

--function OnUnitCreated( playerID, unitID, hexVec, unitType, cultureType, civID, primaryColor, secondaryColor, unitFlagIndex, fogState, selected, military, notInvisible )
--Events.SerialEventUnitCreated.Add( OnUnitCreated )


--------------------------------------------------------------
-- units Max Health
--------------------------------------------------------------

-- Not used, main problem is number of figures relative to global maxHP and the need to pass unitData to UnitFlagManager.lua
function GetUnitMaxHP(unit)
	local unitKey = GetUnitKey ( unit:GetID(), unit:GetOwner())
	return unitData[unitKey].MaxHP or MAX_HP 
end